@@ -1,5 +1,5 @@
 package ZMQ::Raw::Loop::Timer;
-$ZMQ::Raw::Loop::Timer::VERSION = '0.28';
+$ZMQ::Raw::Loop::Timer::VERSION = '0.29';
 use strict;
 use warnings;
 use Scalar::Util qw/weaken/;
@@ -13,6 +13,7 @@ BEGIN
 {
 	@attributes = qw/
 		timer
+		on_cancel
 		on_timeout
 	/;
 
@@ -34,7 +35,7 @@ ZMQ::Raw::Loop::Timer - Timer class
 
 =head1 VERSION
 
-version 0.28
+version 0.29
 
 =head1 DESCRIPTION
 
@@ -59,6 +60,10 @@ B<WARNING>: The API of this module is unstable and may change without warning
 			print "Timed out!\n";
 			$loop->terminate();
 		},
+		on_cancel => sub
+		{
+			print "Cancelled!\n";
+		},
 	);
 
 	$loop->add ($timer);
@@ -102,11 +107,17 @@ sub new
 		croak "on_timeout not a code ref";
 	}
 
+	if ($args{on_cancel} && ref ($args{on_cancel}) ne 'CODE')
+	{
+		croak "on_cancel not a code ref";
+	}
+
 	my $class = ref ($this) || $this;
 	my $self =
 	{
 		timer => $args{timer},
 		on_timeout => $args{on_timeout},
+		on_cancel => $args{on_cancel},
 	};
 
 	return bless $self, $class;
@@ -138,6 +149,11 @@ sub cancel
 	if ($this->loop)
 	{
 		$this->loop->remove ($this);
+
+		if ($this->on_cancel)
+		{
+			&{$this->on_cancel}();
+		}
 	}
 }
 
@@ -174,7 +190,7 @@ sub running
 	return $this->timer->running;
 }
 
-=for Pod::Coverage timer loop on_timeout
+=for Pod::Coverage timer loop on_cancel on_timeout
 
 =head1 AUTHOR
 
